SystemOrganization addCategory: #DefragFAT!

TestCase subclass: #DFDefragAppOneFileTest
	instanceVariableNames: 'app file'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DefragFAT'!

!DFDefragAppOneFileTest methodsFor: 'as yet unclassified' stamp: 'hjo 4/24/2011 07:01'!
setUp

	|pc|
	pc := DFComputer new.
	pc doInstallMemorySize: 3.
	pc doInstallHardDiskSize: 40.
	pc hardDisk doFormatFAT.
	file := pc hardDisk fileSystem doAddFile.
	file doAddFragmentFromIndex: 20 withBytes: 'Hel'.
	file doAddFragmentFromIndex: 3 withBytes: 'lo W'.
	file doAddFragmentFromIndex: 10 withBytes: 'orld!!!!'.
	app := DFDefragApp new.
	app computer: pc.
! !

!DFDefragAppOneFileTest methodsFor: 'as yet unclassified' stamp: 'hjo 4/18/2011 00:19'!
testDefrag

	self assert: 3 equals: file fragCount.
	app doDefrag.
	self assert: 1 equals: file fragCount.! !

!DFDefragAppOneFileTest methodsFor: 'as yet unclassified' stamp: 'hjo 4/24/2011 12:49'!
testOp1MoveFragmentToEndOfDisk
	self assert: 3 equals: file fragments second startingIndex.
	app moveToLastEmptySpaceWithFragmentNumber: 2.
	self assert: 37 equals: file fragments second startingIndex.
	self assert: 'Hello World!!!!' equals: file contents asString! !

!DFDefragAppOneFileTest methodsFor: 'as yet unclassified' stamp: 'hjo 4/24/2011 12:49'!
testOp2MoveFragmentToBeginOfDisk
	app moveToLastEmptySpaceWithFragmentNumber: 2.
	self assert: 20 equals: file fragments first startingIndex.
	app moveToBeginForDiskWithFragmentNumber: 1.
	self assert: 1 equals: file fragments first startingIndex.
	self assert: 'Hello World!!!!' equals: file contents asString! !

!DFDefragAppOneFileTest methodsFor: 'as yet unclassified' stamp: 'hjo 4/24/2011 12:49'!
testOp3MoveFragmentToBeginOfDisk
	app moveToLastEmptySpaceWithFragmentNumber: 2.
	app moveToBeginForDiskWithFragmentNumber: 1.
	self assert: 37 equals: file fragments second startingIndex.
	app moveToBeginForDiskWithFragmentNumber: 2.
	self assert: 4 equals: file fragments second startingIndex.
	self assert: 'Hello World!!!!' equals: file contents asString! !

!DFDefragAppOneFileTest methodsFor: 'as yet unclassified' stamp: 'hjo 4/24/2011 12:49'!
testOp4MoveFragmentToBeginOfDisk
	app moveToLastEmptySpaceWithFragmentNumber: 2.
	app moveToBeginForDiskWithFragmentNumber: 1.
	app moveToBeginForDiskWithFragmentNumber: 2.
	self assert: 10 equals: file fragments third startingIndex.
	app moveToBeginForDiskWithFragmentNumber: 3.
	self assert: 8 equals: file fragments third startingIndex.
	self assert: 'Hello World!!!!' equals: file contents asString! !

!DFDefragAppOneFileTest methodsFor: 'as yet unclassified' stamp: 'hjo 4/24/2011 12:49'!
testOp5UnifyToOneFragment
	app moveToLastEmptySpaceWithFragmentNumber: 2.
	app moveToBeginForDiskWithFragmentNumber: 1.
	app moveToBeginForDiskWithFragmentNumber: 2.
	app moveToBeginForDiskWithFragmentNumber: 3.
	self assert: 3 equals: file fragCount.
	app doUnifyToOneFragment.
	self assert: 1 equals: file fragCount.
	self assert: 'Hello World!!!!' equals: file contents asString! !

!DFDefragAppOneFileTest methodsFor: 'as yet unclassified' stamp: 'hjo 4/24/2011 12:13'!
testSetUp

	self assert: 'Hello World!!!!' equals: file contents asString.! !

TestCase subclass: #DFDiskFragmentTest
	instanceVariableNames: 'hardDisk fat fragments'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DefragFAT'!

!DFDiskFragmentTest methodsFor: 'as yet unclassified' stamp: 'hjo 4/23/2011 18:46'!
setUp

	super setUp.
	hardDisk := DFHardDisk new.
	hardDisk doAllocateSize: 30.
	fat := DFFAT new.
	fat hardDisk: hardDisk.
	fragments := DFDiskFragment new.
	fragments fileSystem: fat.! !

!DFDiskFragmentTest methodsFor: 'as yet unclassified' stamp: 'hjo 4/23/2011 18:46'!
testFATAccessBytes

	fragments startingIndex: 10.
	fragments doWriteBytes: (ByteArray withAll: 'hello world, baby').
	! !

!DFDiskFragmentTest methodsFor: 'as yet unclassified' stamp: 'hjo 4/23/2011 18:46'!
testFATAccessBytesErrorIfTooLong

	fragments startingIndex: 25.
	self should: [fragments doWriteBytes: (ByteArray withAll: 'hello world, baby')] raise: Error  
	.
	! !

!DFDiskFragmentTest methodsFor: 'as yet unclassified' stamp: 'hjo 4/23/2011 18:46'!
testFATAccessErrorIfStartingIndexTooBig

	self should: [ fragments startingIndex: 40] raise: Error.
	  
	! !

!DFDiskFragmentTest methodsFor: 'as yet unclassified' stamp: 'hjo 4/23/2011 18:46'!
testFATAccessErrorIfStartingIndexTooSmall

	self should: [ fragments startingIndex: -3] raise: Error.
	  
	! !

!DFDiskFragmentTest methodsFor: 'as yet unclassified' stamp: 'hjo 4/23/2011 18:47'!
testFATAccessGoodStartingIndex

	self shouldnt: [ fragments startingIndex: 1 ] raise: Error.
	  
	! !

!DFDiskFragmentTest methodsFor: 'as yet unclassified' stamp: 'hjo 4/23/2011 18:46'!
testRead

	| data |
	data := ByteArray withAll: 'hello world, baby'.
	fragments startingIndex: 10.
	fragments doWriteBytes: data.
	self assert: data equals: fragments bytes! !

!DFDiskFragmentTest methodsFor: 'as yet unclassified' stamp: 'hjo 4/23/2011 18:46'!
testWrite

	| before after getBytes getFront getMiddle getBack |
	getBytes := [hardDisk diskBytes copy].
	before := getBytes value.
	fragments startingIndex: 10.
	fragments doWriteBytes: (ByteArray withAll: 'hello world, baby').
	after := getBytes value.
	
	getFront := [:bytes| bytes copyFrom:1 to:9 ].
	getMiddle := [:bytes| bytes copyFrom:10 to: 26 ].
	getBack := [:bytes| bytes copyFrom:27 to: bytes size ].
	self assert: (getFront value: before) = (getFront value: after).
	self deny:  (getMiddle value: before) = (getMiddle value: after).
	self assert: 'hello world, baby' equals: (getMiddle value: after) asString.
	self assert: (getBack value: before) = (getBack value: after).
! !

TestCase subclass: #DFFATTest
	instanceVariableNames: 'fat hardDisk'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DefragFAT'!

!DFFATTest methodsFor: 'as yet unclassified' stamp: 'hjo 4/17/2011 21:39'!
setUp

	super setUp.
	hardDisk := DFHardDisk new.
	hardDisk doAllocateSize: 50.
	hardDisk doFormatFAT.
	fat := hardDisk fileSystem.  ! !

!DFFATTest methodsFor: 'as yet unclassified' stamp: 'hjo 4/17/2011 23:33'!
testDoAddFile

	fat doAddFile.
	self assert: 1 equals: fat files size! !

!DFFATTest methodsFor: 'as yet unclassified' stamp: 'hjo 4/17/2011 21:41'!
testEmpty

	|bytes|
	bytes := ByteArray withAll: #(3 5 10).
	self deny: (fat canPut: bytes startingAt: 0).
	self assert: (fat canPut: bytes startingAt: 25).
	self assert: (fat canPut: bytes startingAt: 48).
	self deny: (fat canPut: bytes startingAt: 49).! !

!DFFATTest methodsFor: 'as yet unclassified' stamp: 'hjo 4/23/2011 18:52'!
testOneAlloc

	|file bytes|
	file := fat doAddFile.
	file doAddFragmentFromIndex: 15 withBytes: (ByteArray withAll: 'i am an headache for you'). 
	bytes := ByteArray withAll: #(3 5 10).
	self deny: (fat canPut: bytes startingAt: 0).
	self assert: (fat canPut: bytes startingAt: 12).
	self deny: (fat canPut: bytes startingAt: 13).
	self deny: (fat canPut: bytes startingAt: 30).
	self deny: (fat canPut: bytes startingAt: 38).
	self assert: (fat canPut: bytes startingAt: 39).
	self assert: (fat canPut: bytes startingAt: 48).
	self deny: (fat canPut: bytes startingAt: 49).
	! !

Object subclass: #DFComputer
	instanceVariableNames: 'memory disk'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DefragFAT'!

!DFComputer methodsFor: 'as yet unclassified' stamp: 'hjo 4/17/2011 17:42'!
doInstallHardDiskSize: anInteger

	self assert: disk isNil.
	disk := DFHardDisk new.
	disk doAllocateSize: anInteger.  ! !

!DFComputer methodsFor: 'as yet unclassified' stamp: 'hjo 4/17/2011 17:43'!
doInstallMemorySize: anInteger 

	self assert: memory isNil.
	memory := DFMemory new.
	memory doInstallSize: anInteger! !

!DFComputer methodsFor: 'as yet unclassified' stamp: 'hjo 4/17/2011 15:22'!
hardDisk

	^disk! !

!DFComputer methodsFor: 'as yet unclassified' stamp: 'hjo 4/18/2011 00:02'!
memory

	^ memory! !

Object subclass: #DFDefragApp
	instanceVariableNames: 'computer'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DefragFAT'!

!DFDefragApp methodsFor: 'as yet unclassified' stamp: 'hjo 4/24/2011 07:01'!
computer: aDFIBMPC 

	computer := aDFIBMPC! !

!DFDefragApp methodsFor: 'as yet unclassified' stamp: 'hjo 4/24/2011 07:00'!
doDefrag

	computer hardDisk fileSystem files do: [:each| self doDefragFor: each]! !

!DFDefragApp methodsFor: 'as yet unclassified' stamp: 'hjo 4/24/2011 12:52'!
doDefragFor: aDFFile 

	computer memory reset.
	self moveToLastEmptySpaceWithFragmentNumber: 2.
	self moveToBeginForDiskWithFragmentNumber: 1.
	self moveToBeginForDiskWithFragmentNumber: 2.
	self moveToBeginForDiskWithFragmentNumber: 3.
	self doUnifyToOneFragment.
! !

!DFDefragApp methodsFor: 'as yet unclassified' stamp: 'hjo 4/24/2011 11:59'!
doUnifyToOneFragment

	self flag: 'rename this starting with "do"'.
	computer hardDisk fileSystem files do: [:each| each doUnifyToOneFragment]! !

!DFDefragApp methodsFor: 'as yet unclassified' stamp: 'hjo 4/24/2011 11:58'!
firstFile
	^ computer hardDisk fileSystem files first! !

!DFDefragApp methodsFor: 'as yet unclassified' stamp: 'hjo 4/24/2011 12:02'!
firstFileGetFragmentNumber: anInteger
	^ self firstFile fragments at: anInteger! !

!DFDefragApp methodsFor: 'as yet unclassified' stamp: 'hjo 4/24/2011 12:18'!
moveFirstFragmentToLastEmptySpaceFor: aDFFile

	aDFFile fragmentOfLeastStartingIndex moveToSectorIndex: 37.
"	(aDFFile fragmentOfLeastStartingIndex) startingIndex: 37"! !

!DFDefragApp methodsFor: 'as yet unclassified' stamp: 'hjo 4/24/2011 12:26'!
moveToBeginForDiskWithFragmentNumber:  anInteger

	| fragmentToMove |
	fragmentToMove := self firstFileGetFragmentNumber: anInteger.
	anInteger = 1 ifTrue: [	^fragmentToMove moveToSectorIndex:  1 ].
	anInteger = 2 ifTrue: [^fragmentToMove moveToSectorIndex: 4  ].
	anInteger = 3 ifTrue: [^fragmentToMove moveToSectorIndex: 8  ].
	self halt.  

! !

!DFDefragApp methodsFor: 'as yet unclassified' stamp: 'hjo 4/24/2011 12:49'!
moveToLastEmptySpaceWithFragmentNumber: anObject

	self moveFirstFragmentToLastEmptySpaceFor: (self firstFile)! !

Object subclass: #DFDiskFragment
	instanceVariableNames: 'startingIndex length fileSystem'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DefragFAT'!

!DFDiskFragment methodsFor: 'as yet unclassified' stamp: 'hjo 4/17/2011 23:47'!
allocationCollidesToPut: aByteArray startingAt: anInteger 

	^((anInteger + aByteArray size - 1) < startingIndex or: 
	[
		(startingIndex + length - 1)  < anInteger
	]) not.
! !

!DFDiskFragment methodsFor: 'as yet unclassified' stamp: 'hjo 4/17/2011 19:45'!
bytes

	^fileSystem bytesFrom: startingIndex to: startingIndex + length - 1! !

!DFDiskFragment methodsFor: 'as yet unclassified' stamp: 'hjo 4/17/2011 19:27'!
doWriteBytes: aByteArray 

	fileSystem doWriteBytes: aByteArray withStartingIndex: startingIndex.	
	length := aByteArray size.! !

!DFDiskFragment methodsFor: 'as yet unclassified' stamp: 'hjo 4/24/2011 08:59'!
endingIndex

	^startingIndex + length - 1! !

!DFDiskFragment methodsFor: 'as yet unclassified' stamp: 'hjo 4/17/2011 17:34'!
fileSystem: aFileSystem 

	fileSystem := aFileSystem! !

!DFDiskFragment methodsFor: 'as yet unclassified' stamp: 'hjo 4/24/2011 09:05'!
isAdjacentTo: aDFDiskFragment 

	^self endingIndex + 1 = aDFDiskFragment startingIndex ! !

!DFDiskFragment methodsFor: 'as yet unclassified' stamp: 'hjo 4/24/2011 12:27'!
moveToSectorIndex: anInteger 

	fileSystem doWriteBytesFrom: startingIndex to: self endingIndex toStartingIndex: anInteger.
	self startingIndex: anInteger  ! !

!DFDiskFragment methodsFor: 'as yet unclassified' stamp: 'hjo 4/18/2011 00:10'!
startingIndex

	^ startingIndex! !

!DFDiskFragment methodsFor: 'as yet unclassified' stamp: 'hjo 4/17/2011 17:35'!
startingIndex: anInteger 

	anInteger > fileSystem hardDiskSize ifTrue: [self error: 'starting index greater than harddisk limit'].  	
	anInteger < 1 ifTrue: [self error: 'starting index less than 1'].  	
	startingIndex := anInteger ! !

Object subclass: #DFFAT
	instanceVariableNames: 'files hardDisk'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DefragFAT'!

!DFFAT methodsFor: 'as yet unclassified' stamp: 'hjo 4/17/2011 21:06'!
bytesFrom: integer1 to: integer2 

	^hardDisk bytesFrom: integer1 to: integer2 ! !

!DFFAT methodsFor: 'as yet unclassified' stamp: 'hjo 4/17/2011 23:34'!
canPut: aByteArray startingAt: anInteger 

	(anInteger >= 1 and: [anInteger + aByteArray size - 1 <= hardDisk diskSize])
		ifFalse: [^false].  
	^files allSatisfy: [:f| (f allocationCollidesToPut: aByteArray startingAt: anInteger) not ]! !

!DFFAT methodsFor: 'as yet unclassified' stamp: 'hjo 4/17/2011 23:17'!
doAddFile

	| newFile |
	newFile := DFFile new.
	newFile fileSystem: self.
	files add: newFile.
	^newFile! !

!DFFAT methodsFor: 'as yet unclassified' stamp: 'hjo 4/17/2011 21:06'!
doWriteBytes: aByteString withStartingIndex: anInteger 

	hardDisk doWriteBytes: aByteString withStartingIndex: anInteger! !

!DFFAT methodsFor: 'as yet unclassified' stamp: 'hjo 4/24/2011 12:23'!
doWriteBytesFrom: aSmallInteger to: aSmallInteger2 toStartingIndex: aSmallInteger3 

	| offset |
	offset := 0.
	aSmallInteger to: aSmallInteger2 do: [:sectorIndex| 
		hardDisk writeByteAt: sectorIndex to: aSmallInteger3+ offset.
		offset := offset + 1
		]
	! !

!DFFAT methodsFor: 'as yet unclassified' stamp: 'hjo 4/17/2011 17:02'!
files

	^ files! !

!DFFAT methodsFor: 'as yet unclassified' stamp: 'hjo 4/17/2011 15:59'!
hardDisk: aDFHardDisk 

	hardDisk := aDFHardDisk ! !

!DFFAT methodsFor: 'as yet unclassified' stamp: 'hjo 4/17/2011 16:19'!
hardDiskSize

	^hardDisk diskSize! !

!DFFAT methodsFor: 'as yet unclassified' stamp: 'hjo 4/17/2011 17:07'!
initialize 

	super initialize.
	files := OrderedCollection new.! !

Object subclass: #DFFile
	instanceVariableNames: 'fragments fileSystem'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DefragFAT'!

!DFFile methodsFor: 'as yet unclassified' stamp: 'hjo 4/23/2011 18:50'!
allocationCollidesToPut: aByteArray startingAt: anInteger 

	^fragments anySatisfy: [:each| each allocationCollidesToPut: aByteArray startingAt: anInteger ]! !

!DFFile methodsFor: 'as yet unclassified' stamp: 'hjo 4/24/2011 12:08'!
contents

	^(fragments collect: #bytes) concatenateAll! !

!DFFile methodsFor: 'as yet unclassified' stamp: 'hjo 4/23/2011 18:51'!
doAddFragmentFromIndex: anInteger withBytes: aByteArray 

	| bytes |
	bytes := DFDiskFragment new.
	bytes fileSystem: fileSystem.
	bytes startingIndex: anInteger.
	bytes doWriteBytes: aByteArray.
	
	fragments add: bytes.! !

!DFFile methodsFor: 'as yet unclassified' stamp: 'hjo 4/23/2011 18:50'!
doDeleteFile

	fragments := OrderedCollection new.! !

!DFFile methodsFor: 'as yet unclassified' stamp: 'hjo 4/24/2011 09:17'!
doUnifyToOneFragment

	| unifiedFragment firstIndex concatenatedBytes |
	self assert: (fragments allSatisfyCouple: [:aFrag :nextFrag| 
		aFrag isAdjacentTo: nextFrag]).
	
	firstIndex := fragments first startingIndex.
	concatenatedBytes := (fragments collect: #bytes) concatenateAll.
	fragments := OrderedCollection new.
	self doAddFragmentFromIndex: firstIndex withBytes: concatenatedBytes.! !

!DFFile methodsFor: 'as yet unclassified' stamp: 'hjo 4/17/2011 17:19'!
fileSystem: aDFFAT 

	fileSystem := aDFFAT! !

!DFFile methodsFor: 'as yet unclassified' stamp: 'hjo 4/23/2011 18:50'!
fragCount

	^fragments size! !

!DFFile methodsFor: 'as yet unclassified' stamp: 'hjo 4/24/2011 12:05'!
fragmentOfLeastStartingIndex
	^ (self fragments asSortedCollection: [ :a :b | a startingIndex <= b startingIndex ]) first! !

!DFFile methodsFor: 'as yet unclassified' stamp: 'hjo 4/23/2011 18:50'!
fragments

	^fragments! !

!DFFile methodsFor: 'as yet unclassified' stamp: 'hjo 4/23/2011 18:50'!
initialize 

	super initialize.
	fragments := OrderedCollection new.! !

Object subclass: #DFHardDisk
	instanceVariableNames: 'diskBytes fileSystem'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DefragFAT'!

!DFHardDisk methodsFor: 'as yet unclassified' stamp: 'hjo 4/17/2011 21:04'!
bytesFrom: integer1 to: integer2 

	^diskBytes copyFrom: integer1 to: integer2! !

!DFHardDisk methodsFor: 'as yet unclassified' stamp: 'hjo 4/17/2011 19:37'!
diskBytes

	^ diskBytes! !

!DFHardDisk methodsFor: 'as yet unclassified' stamp: 'hjo 4/17/2011 16:20'!
diskSize

	^diskBytes size! !

!DFHardDisk methodsFor: 'as yet unclassified' stamp: 'hjo 4/17/2011 21:04'!
doAllocateSize: anInteger 

	self assert: diskBytes isNil.
	diskBytes := ByteArray new: anInteger.! !

!DFHardDisk methodsFor: 'as yet unclassified' stamp: 'hjo 4/17/2011 17:14'!
doFormatFAT

	fileSystem := DFFAT new.
	fileSystem hardDisk: self.
! !

!DFHardDisk methodsFor: 'as yet unclassified' stamp: 'hjo 4/17/2011 21:05'!
doWriteBytes: aString withStartingIndex: anInteger 

	diskBytes replaceFrom: anInteger to: anInteger + aString size - 1 with: aString! !

!DFHardDisk methodsFor: 'as yet unclassified' stamp: 'hjo 4/17/2011 17:12'!
fileSystem

	^ fileSystem! !

!DFHardDisk methodsFor: 'as yet unclassified' stamp: 'hjo 4/17/2011 17:07'!
initialize 

	super initialize.! !

!DFHardDisk methodsFor: 'as yet unclassified' stamp: 'hjo 4/24/2011 12:24'!
writeByteAt: aSmallInteger to: aSmallInteger2 

	diskBytes at: aSmallInteger2 put: (diskBytes at: aSmallInteger)! !

Object subclass: #DFMemory
	instanceVariableNames: 'byteArray'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DefragFAT'!

!DFMemory methodsFor: 'as yet unclassified' stamp: 'hjo 4/17/2011 17:40'!
doInstallSize: anInteger
	 
	byteArray := ByteArray new: anInteger! !

!DFMemory methodsFor: 'as yet unclassified' stamp: 'hjo 4/18/2011 00:03'!
reset

	self flag: 'i dont know what to here yet'! !
